use crate::{open_rwlocked, CvfsResult};
use multi_key_manager::file::File;
use std::{fs::remove_file, path::PathBuf};

#[derive(Debug)]
pub enum FileOrPath {
    File(PathBuf, File),
    Path(PathBuf),
}

impl FileOrPath {
    pub fn exists(&self) -> bool {
        match self {
            FileOrPath::File(_, _) => true,
            FileOrPath::Path(_) => false,
        }
    }

    pub fn borrow_or_create_file(&mut self) -> CvfsResult<&mut File> {
        if let FileOrPath::Path(real_path) = self {
            let real_file = open_rwlocked(&real_path)?;
            *self = FileOrPath::File(real_path.clone(), real_file);
        }
        if let FileOrPath::File(_, file) = self {
            Ok(file)
        } else {
            unreachable!("changed type variant before")
        }
    }

    pub fn delete_if_exists(&mut self) -> CvfsResult<()> {
        if let FileOrPath::File(real_path, _real_file) = self {
            // just hope that the compile optimises this line even more
            *self = FileOrPath::Path(real_path.clone());
        }

        if let FileOrPath::Path(real_path) = self {
            if real_path.exists() {
                remove_file(&real_path)?;
            }
        }
        Ok(())
    }
}
